using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI;
using System.IO;

namespace VSLiveBoston.WebParts.Common
{
	public class CustomWebPartChrome : WebPartChrome
	{
		private CustomWebPartZone m_zone;

		public CustomWebPartChrome(CustomWebPartZone zone, WebPartManager manager) 
			: base(zone, manager)
		{
			m_zone = zone;
		}

		public override void RenderWebPart(
			System.Web.UI.HtmlTextWriter writer, WebPart webPart)
		{
			string output = m_zone.LayoutTemplate;

			output = output.Replace("%%TITLE%%", webPart.Title);

			output = output.Replace("%%VERB_CLOSE%%",
				RenderLinkButton(webPart, m_zone.CloseVerb.Text, "Close"));

			output = output.Replace("%%VERB_DELETE%%",
				RenderLinkButton(webPart, m_zone.DeleteVerb.Text, "Delete"));

			output = output.Replace("%%VERB_MENU%%",
				RenderVerbs(webPart));
					

			if (webPart.ChromeState == PartChromeState.Normal)
			{
				output = output.Replace("%%WEBPART_CONTENT%%",
					RenderPartContents(webPart));

				output = output.Replace("%%VERB_MINIMIZE%%",
					RenderLinkButton(webPart, m_zone.MinimizeVerb.Text, "Minimize"));
			}
			else if (webPart.ChromeState == PartChromeState.Minimized)
			{
				output = output.Replace("%%WEBPART_CONTENT%%",
					String.Empty);

				output = output.Replace("%%VERB_MINIMIZE%%",
					RenderLinkButton(webPart, m_zone.RestoreVerb.Text, "Restore"));
			}

			writer.WriteLine(output.ToString());
		}

		private string RenderVerbs(WebPart webPart)
		{
			StringBuilder output = new StringBuilder();
			string template = m_zone.LayoutVerbTemplate;

			foreach (WebPartVerb verb in webPart.Verbs)
			{
				output.Append(
					template.Replace("%%LINK_BUTTON%%",
					RenderLinkButton(webPart, verb.Text, verb.ID)));
			}

			return output.ToString();
		}

		private string RenderLinkButton(WebPart part, string text, string id)
		{
			StringBuilder sb = new StringBuilder();
			HtmlTextWriter writer = new HtmlTextWriter(new StringWriter(sb));

			VerbLinkButton button = new VerbLinkButton( 
				this.Zone, String.Format("{0}:{1}", part.ID, id));

			button.ID = id;
			button.Text = text;
			button.Enabled = true;
			button.Page = this.Zone.Page;

			button.RenderControl(writer);

			return sb.ToString();
		}

		private string RenderPartContents(WebPart webPart)
		{
			StringBuilder output = new StringBuilder();
			HtmlTextWriter writer = new HtmlTextWriter(new StringWriter(output));

			RenderPartContents(writer, webPart);

			return output.ToString();
		}

	}
}
